@@ -16,6 +16,7 @@
 #         详细的授权流程，请联系 public@ricequant.com 获取。
 import os
 import pickle
+from functools import lru_cache
 from datetime import date, datetime, timedelta
 from itertools import groupby
 from typing import Dict, Iterable, List, Optional, Sequence, Union
@@ -32,11 +33,11 @@ from rqalpha.utils.functools import lru_cache
 from rqalpha.utils.typing import DateLike
 from rqalpha.environment import Environment
 
-from .adjust import FIELDS_REQUIRE_ADJUSTMENT, adjust_bars
-from .storage_interface import (AbstractCalendarStore, AbstractDateSet,
+from rqalpha.data.base_data_source.adjust import FIELDS_REQUIRE_ADJUSTMENT, adjust_bars
+from rqalpha.data.base_data_source.storage_interface import (AbstractCalendarStore, AbstractDateSet,
                                 AbstractDayBarStore, AbstractDividendStore,
                                 AbstractInstrumentStore)
-from .storages import (DateSet, DayBarStore, DividendStore,
+from rqalpha.data.base_data_source.storages import (DateSet, DayBarStore, DividendStore,
                        ExchangeTradingCalendarStore, FutureDayBarStore,
                        FutureInfoStore, InstrumentStore,
                        ShareTransformationStore, SimpleFactorStore,
@@ -148,6 +149,7 @@ class BaseDataSource(AbstractDataSource):
         # type: (AbstractDateSet) -> None
         self._suspend_days.append(date_set)
 
+    @lru_cache(2048)
     def get_dividend(self, instrument):
         try:
             dividend_store = self._dividends[instrument.type]
@@ -330,6 +332,7 @@ class BaseDataSource(AbstractDataSource):
     def current_snapshot(self, instrument, frequency, dt):
         raise NotImplementedError
 
+    @lru_cache(2048)
     def get_split(self, instrument):
         try:
             splilt_store = self._split_factors[instrument.type]
