@@ -274,9 +274,7 @@ class GenerateFileTask(ProgressedTask):
 
 STOCK_FIELDS = ['open', 'close', 'high', 'low', 'limit_up', 'limit_down', 'volume', 'total_turnover']
 INDEX_FIELDS = ['open', 'close', 'high', 'low', 'volume', 'total_turnover']
-# FUTURES_FIELDS = STOCK_FIELDS + ['basis_spread', 'settlement', 'prev_settlement']
-FUTURES_FIELDS = STOCK_FIELDS + ['settlement', 'prev_settlement']
-# FUND_FIELDS = STOCK_FIELDS + ['acc_net_value', 'unit_net_value', 'discount_rate']
+FUTURES_FIELDS = STOCK_FIELDS + ['settlement', 'prev_settlement', 'open_interest']
 FUND_FIELDS = STOCK_FIELDS
 
 
@@ -316,36 +314,69 @@ class GenerateDayBarTask(DayBarTask):
 
 
 class UpdateDayBarTask(DayBarTask):
+    def h5_has_valid_fields(self, h5, wanted_fields):
+        obid_gen = (k for k in h5.keys())
+        wanted_fields = set(wanted_fields)
+        wanted_fields.add('datetime')
+        try:
+            h5_fields = set(h5[next(obid_gen)].dtype.fields.keys())
+        except StopIteration:
+            pass
+        else:
+            return h5_fields == wanted_fields
+        return False
+
     def __call__(self, path, fields, **kwargs):
-        with h5py.File(path, 'a') as h5:
-            for order_book_id in self._order_book_ids:
-                if order_book_id in h5:
-                    try:
-                        start_date = rqdatac.get_next_trading_date(int(h5[order_book_id]['datetime'][-1] // 1000000))
-                    except ValueError:
-                        h5.pop(order_book_id)
-                        start_date = START_DATE
-                else:
-                    start_date = START_DATE
-                df = rqdatac.get_price(order_book_id, start_date, END_DATE, '1d',
-                                       adjust_type='none', fields=fields, expect_df=True)
-                if not (df is None or df.empty):
-                    df = df[fields]  # Future order_book_id like SC888 will auto add 'dominant_id'
-                    df = df.loc[order_book_id]
-                    df.reset_index(inplace=True)
-                    df['datetime'] = [convert_date_to_int(d) for d in df['date']]
-                    del df['date']
-                    df.set_index('datetime', inplace=True)
+        need_recreate_h5 = False
+        try:
+            with h5py.File(path, 'r') as h5:
+                need_recreate_h5 = not self.h5_has_valid_fields(h5, fields)
+        except (OSError, RuntimeError):
+            need_recreate_h5 = True
+        if need_recreate_h5:
+            yield from GenerateDayBarTask(self._order_book_ids)(path, fields, **kwargs)
+        else:
+            try:
+                h5 = h5py.File(path, 'a')
+            except OSError:
+                raise OSError("File {} update failed, if it is using, please update later, "
+                              "or you can delete then update again".format(path))
+            try:
+                for order_book_id in self._order_book_ids:
                     if order_book_id in h5:
-                        data = np.array(
-                            [tuple(i) for i in chain(h5[order_book_id][:], df.to_records())],
-                            dtype=h5[order_book_id].dtype
-                        )
-                        del h5[order_book_id]
-                        h5.create_dataset(order_book_id, data=data, **kwargs)
+                        try:
+                            last_date = int(h5[order_book_id]['datetime'][-1] // 1000000)
+                        except OSError:
+                            raise OSError("File {} update failed, if it is using, please update later, "
+                                          "or you can delete then update again".format(path))
+                        except ValueError:
+                            h5.pop(order_book_id)
+                            start_date = START_DATE
+                        else:
+                            start_date = rqdatac.get_next_trading_date(last_date)
                     else:
-                        h5.create_dataset(order_book_id, data=df.to_records(), **kwargs)
-                yield 1
+                        start_date = START_DATE
+                    df = rqdatac.get_price(order_book_id, start_date, END_DATE, '1d',
+                                        adjust_type='none', fields=fields, expect_df=True)
+                    if not (df is None or df.empty):
+                        df = df[fields]  # Future order_book_id like SC888 will auto add 'dominant_id'
+                        df = df.loc[order_book_id]
+                        df.reset_index(inplace=True)
+                        df['datetime'] = [convert_date_to_int(d) for d in df['date']]
+                        del df['date']
+                        df.set_index('datetime', inplace=True)
+                        if order_book_id in h5:
+                            data = np.array(
+                                [tuple(i) for i in chain(h5[order_book_id][:], df.to_records())],
+                                dtype=h5[order_book_id].dtype
+                            )
+                            del h5[order_book_id]
+                            h5.create_dataset(order_book_id, data=data, **kwargs)
+                        else:
+                            h5.create_dataset(order_book_id, data=df.to_records(), **kwargs)
+                    yield 1
+            finally:
+                h5.close()
 
 
 def init_rqdatac_with_warnings_catch():
