@@ -16,14 +16,12 @@
 #         详细的授权流程，请联系 public@ricequant.com 获取。
 
 import datetime
-import math
 from decimal import Decimal, getcontext
 from itertools import chain
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, Union, Tuple
 
 import numpy as np
 import pandas as pd
-import six
 from rqalpha.api import export_as_api
 from rqalpha.apis.api_abstract import (order, order_percent, order_shares,
                                        order_target_percent,
@@ -71,15 +69,16 @@ def _get_account_position_ins(id_or_ins):
     return account, position, ins
 
 
-def _get_ksh_amount(amount):
-    return 0 if abs(amount) < KSH_MIN_AMOUNT else amount // 1
+def _round_order_quantity(ins, quantity) -> int:
+    if ins.type == "CS" and ins.board_type == "KSH":
+        # KSH can buy(sell) 201, 202 shares
+        return 0 if abs(quantity) < KSH_MIN_AMOUNT else int(quantity)
+    else:
+        round_lot = ins.round_lot
+        return int(Decimal(quantity) / Decimal(round_lot)) * round_lot
 
 
-def _is_ksh(ins):
-    return ins.type == "CS" and ins.board_type == "KSH"
-
-
-def _submit_order(ins, amount, side, position_effect, style, quantity, auto_switch_order_value):
+def _submit_order(ins, amount, side, position_effect, style, current_quantity, auto_switch_order_value):
     env = Environment.get_instance()
     if isinstance(style, LimitOrder):
         if not is_valid_price(style.get_limit_price()):
@@ -89,15 +88,9 @@ def _submit_order(ins, amount, side, position_effect, style, quantity, auto_swit
         user_system_log.warn(
             _(u"Order Creation Failed: [{order_book_id}] No market data").format(order_book_id=ins.order_book_id))
         return
-    round_lot = int(ins.round_lot)
 
-    if side in [SIDE.BUY, side.SELL]:
-        if not (side == SIDE.SELL and quantity == abs(amount)):
-            if _is_ksh(ins):
-                # KSH can buy(sell) 201, 202 shares
-                amount = _get_ksh_amount(amount)
-            else:
-                amount = int(Decimal(amount) / Decimal(round_lot)) * round_lot
+    if (side == SIDE.BUY) or (side == SIDE.SELL and current_quantity != abs(amount)):
+        amount = _round_order_quantity(ins, amount)
 
     if amount == 0:
         user_system_log.warn(_(
@@ -109,14 +102,12 @@ def _submit_order(ins, amount, side, position_effect, style, quantity, auto_swit
         order.set_frozen_price(price)
     if side == SIDE.BUY and auto_switch_order_value:
         account, position, ins = _get_account_position_ins(ins)
-        if not is_cash_enough(env, order, account):
+        if not is_cash_enough(env, order, account.cash):
             user_system_log.warn(_(
                 "insufficient cash, use all remaining cash({}) to create order"
             ).format(account.cash))
             return _order_value(account, position, ins, account.cash, style)
-    if env.can_submit_order(order):
-        env.broker.submit_order(order)
-        return order
+    return env.submit_order(order)
 
 
 def _order_shares(ins, amount, style, quantity, auto_switch_order_value):
@@ -139,21 +130,16 @@ def _order_value(account, position, ins, cash_amount, style):
             return
 
     amount = int(Decimal(cash_amount) / Decimal(price))
-
-    min_round_lot = int(ins.round_lot)
+    round_lot = int(ins.round_lot)
     if cash_amount > 0:
-        if _is_ksh(ins):
-            amount = _get_ksh_amount(amount)
-            min_round_lot = 1
-        else:
-            amount = int(Decimal(amount) / Decimal(min_round_lot)) * min_round_lot
+        amount = _round_order_quantity(ins, amount)
         while amount > 0:
             expected_transaction_cost = env.get_order_transaction_cost(Order.__from_create__(
                 ins.order_book_id, amount, SIDE.BUY, LimitOrder(price), POSITION_EFFECT.OPEN
             ))
             if amount * price + expected_transaction_cost <= cash_amount:
                 break
-            amount -= min_round_lot
+            amount -= round_lot
         else:
             user_system_log.warn(_(
                 u"Order Creation Failed: 0 order quantity, order_book_id={order_book_id}"
@@ -262,6 +248,10 @@ def order_lots(id_or_ins, amount, price=None, style=None):
                          auto_switch_order_value)
 
 
+ORDER_TARGET_PORTFOLIO_SUPPORTED_INS_TYPES = {
+    INSTRUMENT_TYPE.CS, INSTRUMENT_TYPE.ETF, INSTRUMENT_TYPE.LOF, INSTRUMENT_TYPE.INDX}
+
+
 @export_as_api
 @ExecutionContext.enforce_phase(
     EXECUTION_PHASE.OPEN_AUCTION,
@@ -270,12 +260,17 @@ def order_lots(id_or_ins, amount, price=None, style=None):
     EXECUTION_PHASE.SCHEDULED,
     EXECUTION_PHASE.GLOBAL
 )
-def order_target_portfolio(target_portfolio):
-    # type: (Dict[Union[str, Instrument], float]) -> List[Order]
+def order_target_portfolio(target_portfolio: Dict[Union[str, Instrument], Union[float, Tuple[float, float]]]) -> List[Order]:
     """
-    买入/卖出证券以批量调整证券的仓位，以期使其持仓市值占账户总权益的比重达到指定值。
+    批量调整股票仓位至目标权重。注意：股票账户中未出现在 target_portfolio 中的资产将被平仓！
 
-    :param target_portfolio: 下单标的物及其目标市值占比的字典
+    该 API 的参数 target_portfolio 为字典，key 为 order_book_id 或 instrument，value 有两种数据类型可选：
+
+      * value 为权重。此时将根据股票最新价计算目标持仓数量并发出市价单调仓。
+
+      * value 为权重和价格组成的 tuple。此时将根据该价格计算目标权重并发出限价单（Signal 模式下将使用该价格撮合）。
+
+    :param target_portfolio: 目标权重字典，key 为 order_book_id，value 为权重或权重和价格组成的 tuple。
 
     :example:
 
@@ -283,80 +278,91 @@ def order_target_portfolio(target_portfolio):
 
         # 调整仓位，以使平安银行和万科 A 的持仓占比分别达到 10% 和 15%
         order_target_portfolio({
-            '000001.XSHE': 0.1
+            '000001.XSHE': 0.1,
             '000002.XSHE': 0.15
         })
+
+        # 调整仓位，分别以 14 和 26 元发出限价单，目标是使平安银行和万科 A 的持仓占比分别达到 10% 和 15%
+        order_target_portfolio({
+            '000001.XSHE': (0.1, 14),
+            '000002.XSHE': (0.15, 26)
+        })
     """
-
-    if isinstance(target_portfolio, pd.Series):
-        # FIXME: kind of dirty
-        total_percent = sum(target_portfolio)
-    else:
-        total_percent = sum(target_portfolio.values())
-    if total_percent > 1 and not np.isclose(total_percent, 1):
-        raise RQInvalidArgument(_(u"total percent should be lower than 1, current: {}").format(total_percent))
-
     env = Environment.get_instance()
-    account = env.portfolio.accounts[DEFAULT_ACCOUNT_TYPE.STOCK]
-    account_value = account.total_value
-    target_quantities = {}
-    for id_or_ins, target_percent in target_portfolio.items():
-        order_book_id = assure_order_book_id(id_or_ins)
-        if target_percent < 0:
-            raise RQInvalidArgument(_(u"target percent of {} should between 0 and 1, current: {}").format(
-                order_book_id, target_percent
-            ))
+    target = {}
+    for id_or_ins, target_quantity_price in target_portfolio.items():
+        ins = assure_instrument(id_or_ins)
+        if not ins:
+            raise RQInvalidArgument(_(
+                "function order_target_portfolio: invalid keys of target_portfolio, "
+                "expected order_book_ids or Instrument objects, got {} (type: {})"
+            ).format(id_or_ins, type(id_or_ins)))
+        if ins.type not in ORDER_TARGET_PORTFOLIO_SUPPORTED_INS_TYPES:
+            raise RQInvalidArgument(_(
+                "function order_target_portfolio: invalid instrument type, excepted CS/ETF/LOF/INDX, got {}"
+            ).format(ins.order_book_id))
+        order_book_id = ins.order_book_id
         price = env.data_proxy.get_last_price(order_book_id)
         if not is_valid_price(price):
             user_system_log.warn(
                 _(u"Order Creation Failed: [{order_book_id}] No market data").format(order_book_id=order_book_id)
             )
             continue
-        target_quantities[order_book_id] = (account_value * target_percent / price, price)
+        try:
+            target_percent, target_price = target_quantity_price
+        except TypeError:
+            target_percent = target_quantity_price
+            target_price = None
+        else:
+            if not is_valid_price(target_price):
+                raise RQInvalidArgument(_(
+                    "function order_target_portfolio: invalid order price {target_price} of {id_or_ins}"
+                ).format(id_or_ins=id_or_ins, target_price=target_price))
+        if target_percent == 0:
+            continue
+        elif target_percent < 0:
+            raise RQInvalidArgument(_(
+                "function order_target_portfolio: invalid values of target_portfolio, "
+                "excepted float between 0 and 1, got {} (key: {})"
+            ).format(target_percent, id_or_ins))
 
-    close_orders, open_orders = [], []
+        target[order_book_id] = target_percent, target_price, price
+    total_percent = sum(p for p, *__ in target.values())
+    if total_percent > 1 and not np.isclose(total_percent, 1):
+        raise RQInvalidArgument(_("total percent should be lower than 1, current: {}").format(total_percent))
+
+    account = env.portfolio.accounts[DEFAULT_ACCOUNT_TYPE.STOCK]
     current_quantities = {
         p.order_book_id: p.quantity for p in account.get_positions() if p.direction == POSITION_DIRECTION.LONG
     }
     for order_book_id, quantity in current_quantities.items():
-        if order_book_id not in target_portfolio:
-            close_orders.append(Order.__from_create__(
+        # 先把不在目标权重中的仓位平掉
+        if order_book_id not in target:
+            env.submit_order(Order.__from_create__(
                 order_book_id, quantity, SIDE.SELL, MarketOrder(), POSITION_EFFECT.CLOSE
             ))
 
-    for order_book_id, (target_quantity, price) in target_quantities.items():
-        ins = assure_instrument(order_book_id)
-        round_lot = 100
-        if order_book_id in current_quantities:
-            delta_quantity = target_quantity - current_quantities[order_book_id]
+    account_value = account.total_value
+    close_orders, open_orders = [], []
+    for order_book_id, (target_percent, target_price, price) in target.items():
+        delta_quantity = (account_value * target_percent / (target_price or price)) - current_quantities.get(order_book_id, 0)
+        delta_quantity = _round_order_quantity(env.data_proxy.instrument(order_book_id), delta_quantity)
+        if delta_quantity == 0:
+            continue
+        elif delta_quantity > 0:
+            quantity, side, position_effect = delta_quantity, SIDE.BUY, POSITION_EFFECT.OPEN
+            order_list = open_orders
         else:
-            delta_quantity = target_quantity
-
-        if _is_ksh(ins):
-            # KSH can buy(sell) 201, 202 shares
-            delta_quantity = _get_ksh_amount(delta_quantity)
+            quantity, side, position_effect = abs(delta_quantity), SIDE.SELL, POSITION_EFFECT.CLOSE
+            order_list = close_orders
+        if target_price:
+            order = Order.__from_create__(order_book_id, quantity, side, LimitOrder(target_price), position_effect)
         else:
-            delta_quantity = int(Decimal(delta_quantity) / Decimal(round_lot)) * round_lot
+            order = Order.__from_create__(order_book_id, quantity, side, MarketOrder(), position_effect)
+            order.set_frozen_price(price)
+        order_list.append(order)
 
-        if delta_quantity > 0:
-            open_order = Order.__from_create__(
-                order_book_id, delta_quantity, SIDE.BUY, MarketOrder(), POSITION_EFFECT.OPEN
-            )
-            open_order.set_frozen_price(price)
-            open_orders.append(open_order)
-        elif delta_quantity < -1:
-            close_order = Order.__from_create__(
-                order_book_id, abs(delta_quantity), SIDE.SELL, MarketOrder(), POSITION_EFFECT.CLOSE
-            )
-            close_order.set_frozen_price(price)
-            close_orders.append(close_order)
-
-    submit_orders = []
-    for order in chain(close_orders, open_orders):
-        if env.can_submit_order(order):
-            submit_orders.append(order)
-            env.broker.submit_order(order)
-    return submit_orders
+    return list(env.submit_order(o) for o in chain(close_orders, open_orders))
 
 
 @export_as_api
