@@ -26,7 +26,7 @@ class IsTradingValidator(AbstractFrontendValidator):
         self._env = env
 
     def can_submit_order(self, order, account=None):
-        instrument = self._env.data_proxy.instruments(order.order_book_id)
+        instrument = self._env.data_proxy.instrument(order.order_book_id)
         if instrument.listed_date > self._env.trading_dt:
             user_system_log.warn(_(u"Order Creation Failed: {order_book_id} is not listed!").format(
                 order_book_id=order.order_book_id,
