@@ -16,7 +16,7 @@
 #         详细的授权流程，请联系 public@ricequant.com 获取。
 
 from itertools import chain
-from typing import Callable, Dict, Iterable, List, Optional, Union
+from typing import Callable, Dict, Iterable, List, Optional, Union, Tuple
 
 import six
 from rqalpha.const import POSITION_DIRECTION, POSITION_EFFECT
@@ -33,7 +33,15 @@ from rqalpha.portfolio.position import Position, PositionProxyDict
 OrderApiType = Callable[[str, Union[int, float], OrderStyle, bool], List[Order]]
 
 
-class Account:
+class AccountMeta(type):
+    def __new__(mcs, *args, **kwargs):
+        cls = type.__new__(mcs, *args, **kwargs)
+        cls._margin = cls.margin
+        cls.margin = property(lambda s: 0)  # black magic: improve performance for pure stock strategy
+        return cls
+
+
+class Account(metaclass=AccountMeta):
     """
     账户，多种持仓和现金的集合。
 
@@ -46,12 +54,11 @@ class Account:
         "dividend_receivable",
     ]
 
-    def __init__(self, type, total_cash, init_positions):
-        # type: (str, float, Dict[str, int]) -> None
-        self._type = type
+    def __init__(self, account_type: str, total_cash: float, init_positions: Dict[str, Tuple[int, Optional[float]]]):
+        self._type = account_type
         self._total_cash = total_cash  # 包含保证金的总资金
 
-        self._positions = {}
+        self._positions: Dict[str, Dict[POSITION_DIRECTION, Position]] = {}
         self._backward_trade_set = set()
         self._frozen_cash = 0
 
@@ -61,9 +68,9 @@ class Account:
         self._management_fee_rate = 0.0
         self._management_fees = 0.0
 
-        for order_book_id, init_quantity in init_positions.items():
+        for order_book_id, (init_quantity, init_price) in init_positions.items():
             position_direction = POSITION_DIRECTION.LONG if init_quantity > 0 else POSITION_DIRECTION.SHORT
-            self._get_or_create_pos(order_book_id, position_direction, init_quantity)
+            self._get_or_create_pos(order_book_id, position_direction, init_quantity, init_price)
 
     def __repr__(self):
         positions_repr = {}
@@ -88,8 +95,8 @@ class Account:
         event_bus.add_listener(EVENT.PRE_BEFORE_TRADING, self._on_before_trading)
         event_bus.add_listener(EVENT.SETTLEMENT, self._on_settlement)
 
-        event_bus.prepend_listener(EVENT.BAR, self._update_last_price)
-        event_bus.prepend_listener(EVENT.TICK, self._update_last_price)
+        event_bus.prepend_listener(EVENT.BAR, self._on_bar)
+        event_bus.prepend_listener(EVENT.TICK, self._on_tick)
 
     def get_state(self):
         return {
@@ -149,14 +156,12 @@ class Account:
                 continue
             yield position
 
-    def get_position(self, order_book_id, direction):
-        # type: (str, POSITION_DIRECTION) -> Position
+    def get_position(self, order_book_id: str, direction: POSITION_DIRECTION = POSITION_DIRECTION.LONG) -> Position:
         """
         获取某个标的的持仓对象
 
         :param order_book_id: 标的编号
         :param direction: 持仓方向
-
         """
         try:
             return self._positions[order_book_id][direction]
@@ -208,12 +213,11 @@ class Account:
         return sum(p.transaction_cost for p in self._iter_pos())
 
     @property
-    def margin(self):
-        # type: () -> float
+    def margin(self) -> float:
         """
         总保证金
         """
-        return sum(p.margin for p in self._iter_pos())
+        return sum(getattr(p, "margin", 0) for p in self._iter_pos())
 
     @property
     def buy_margin(self):
@@ -221,7 +225,7 @@ class Account:
         """
         多方向保证金
         """
-        return sum(p.margin for p in self._iter_pos(POSITION_DIRECTION.LONG))
+        return sum(getattr(p, "margin", 0) for p in self._iter_pos(POSITION_DIRECTION.LONG))
 
     @property
     def sell_margin(self):
@@ -229,7 +233,7 @@ class Account:
         """
         空方向保证金
         """
-        return sum(p.margin for p in self._iter_pos(POSITION_DIRECTION.SHORT))
+        return sum(getattr(p, "margin", 0) for p in self._iter_pos(POSITION_DIRECTION.SHORT))
 
     @property
     def daily_pnl(self):
@@ -355,23 +359,45 @@ class Account:
         else:
             return chain(*[six.itervalues(p) for p in six.itervalues(self._positions)])
 
-    def _get_or_create_pos(self, order_book_id, direction, init_quantity=0):
-        # type: (str, Union[str, POSITION_DIRECTION], Optional[int]) -> Position
+    def _get_or_create_pos(
+            self,
+            order_book_id: str,
+            direction: Union[POSITION_DIRECTION, str],
+            init_quantity: float = 0,
+            init_price : Optional[float] = None
+    ) -> Position:
         if order_book_id not in self._positions:
+            env = Environment.get_instance()
             if direction == POSITION_DIRECTION.LONG:
-                long_init_position, short_init_position = init_quantity, 0
+                long_quantity, short_quantity = init_quantity, 0
             else:
-                long_init_position, short_init_position = 0, init_quantity
-
+                long_quantity, short_quantity = 0, init_quantity
             positions = self._positions.setdefault(order_book_id, {
-                POSITION_DIRECTION.LONG: Position(order_book_id, POSITION_DIRECTION.LONG, long_init_position),
-                POSITION_DIRECTION.SHORT: Position(order_book_id, POSITION_DIRECTION.SHORT, short_init_position)
+                POSITION_DIRECTION.LONG: Position(order_book_id, POSITION_DIRECTION.LONG, long_quantity, init_price),
+                POSITION_DIRECTION.SHORT: Position(order_book_id, POSITION_DIRECTION.SHORT, short_quantity, init_price)
             })
+            if not init_price:
+                last_price = env.get_last_price(order_book_id)
+                for p in positions.values():
+                    p.update_last_price(last_price)
+            if hasattr(positions[direction], "margin") and hasattr(self.__class__, "_margin"):
+                # black magic: improve performance for pure stock strategy
+                setattr(self.__class__, "margin", self.__class__._margin)
+                del self.__class__._margin
         else:
             positions = self._positions[order_book_id]
         return positions[direction]
 
-    def _update_last_price(self, _):
+    def _on_tick(self, event):
+        tick = event.tick
+        try:
+            positions = self._positions[tick.order_book_id]
+        except KeyError:
+            return
+        for position in positions.values():
+            position.update_last_price(tick.last)
+
+    def _on_bar(self, _):
         env = Environment.get_instance()
         for order_book_id, positions in self._positions.items():
             price = env.get_last_price(order_book_id)
@@ -382,7 +408,7 @@ class Account:
     def _frozen_cash_of_order(self, order):
         env = Environment.get_instance()
         if order.position_effect == POSITION_EFFECT.OPEN:
-            instrument = env.data_proxy.instruments(order.order_book_id)
+            instrument = env.data_proxy.instrument(order.order_book_id)
             order_cost = instrument.calc_cash_occupation(order.frozen_price, order.quantity, order.position_direction)
         else:
             order_cost = 0
