@@ -1,19 +1,21 @@
 # -*- coding: utf-8 -*-
-# 版权所有 2019 深圳米筐科技有限公司（下称“米筐科技”）
+# 版权所有 2021 深圳米筐科技有限公司（下称“米筐科技”）
 #
 # 除非遵守当前许可，否则不得使用本软件。
 #
 #     * 非商业用途（非商业用途指个人出于非商业目的使用本软件，或者高校、研究所等非营利机构出于教育、科研等目的使用本软件）：
-#         遵守 Apache License 2.0（下称“Apache 2.0 许可”），您可以在以下位置获得 Apache 2.0 许可的副本：http://www.apache.org/licenses/LICENSE-2.0。
+#         遵守 Apache License 2.0（下称“Apache 2.0 许可”），
+#         您可以在以下位置获得 Apache 2.0 许可的副本：http://www.apache.org/licenses/LICENSE-2.0。
 #         除非法律有要求或以书面形式达成协议，否则本软件分发时需保持当前许可“原样”不变，且不得附加任何条件。
 #
 #     * 商业用途（商业用途指个人出于任何商业目的使用本软件，或者法人或其他组织出于任何目的使用本软件）：
-#         未经米筐科技授权，任何个人不得出于任何商业目的使用本软件（包括但不限于向第三方提供、销售、出租、出借、转让本软件、本软件的衍生产品、引用或借鉴了本软件功能或源代码的产品或服务），任何法人或其他组织不得出于任何目的使用本软件，否则米筐科技有权追究相应的知识产权侵权责任。
+#         未经米筐科技授权，任何个人不得出于任何商业目的使用本软件（包括但不限于向第三方提供、销售、出租、出借、转让本软件、
+#         本软件的衍生产品、引用或借鉴了本软件功能或源代码的产品或服务），任何法人或其他组织不得出于任何目的使用本软件，
+#         否则米筐科技有权追究相应的知识产权侵权责任。
 #         在此前提下，对本软件的使用同样需要遵守 Apache 2.0 许可，Apache 2.0 许可与本许可冲突之处，以本许可为准。
 #         详细的授权流程，请联系 public@ricequant.com 获取。
 
 import os
-import locale
 import codecs
 
 import pandas as pd
@@ -21,11 +23,10 @@ import yaml
 import simplejson as json
 import six
 
-from rqalpha.const import RUN_TYPE, PERSIST_MODE, MARKET, COMMISSION_TYPE
+from rqalpha.const import RUN_TYPE, PERSIST_MODE, COMMISSION_TYPE
 from rqalpha.utils import RqAttrDict, logger
-from rqalpha.utils.i18n import gettext as _, localization
+from rqalpha.utils.i18n import gettext as _, set_locale
 from rqalpha.utils.dict_func import deep_update
-from rqalpha.utils.py2 import to_utf8
 from rqalpha.utils.logger import system_log
 from rqalpha.mod.utils import mod_config_value_parse
 
@@ -111,19 +112,7 @@ def dump_config(config_path, config, dumper=yaml.Dumper):
     if not os.path.exists(dirname):
         os.makedirs(dirname)
     with codecs.open(config_path, mode='w', encoding='utf-8') as stream:
-        stream.write(to_utf8(yaml.dump(config, Dumper=dumper)))
-
-
-def set_locale(lc):
-    # FIXME: It should depends on the system and locale config
-    try:
-        locale.setlocale(locale.LC_ALL, "en_US.UTF-8")
-        locale.setlocale(locale.LC_CTYPE, "en_US.UTF-8")
-        os.environ['TZ'] = 'Asia/Shanghai'
-    except Exception as e:
-        if os.name != 'nt':
-            raise
-    localization.set_locale([lc])
+        stream.write(yaml.dump(config, Dumper=dumper))
 
 
 def parse_config(config_args, config_path=None, click_type=False, source_code=None, user_funcs=None):
@@ -152,7 +141,8 @@ def parse_config(config_args, config_path=None, click_type=False, source_code=No
 
     if click_type:
         for k, v in config_args.items():
-            if v is None:
+            # click multiple=True时传入tuple类型 无输入时为tuple()
+            if v is None or (v == tuple()):
                 continue
             if k == 'base__accounts' and not v:
                 continue
@@ -184,12 +174,11 @@ def parse_config(config_args, config_path=None, click_type=False, source_code=No
     config.base.accounts = parse_accounts(config.base.accounts)
     config.base.init_positions = parse_init_positions(config.base.init_positions)
     config.base.persist_mode = parse_persist_mode(config.base.persist_mode)
-    config.base.market = parse_market(config.base.market)
     config.base.future_info = parse_future_info(config.base.future_info)
 
     if config.extra.context_vars:
         if isinstance(config.extra.context_vars, six.string_types):
-            config.extra.context_vars = json.loads(to_utf8(config.extra.context_vars))
+            config.extra.context_vars = json.loads(config.extra.context_vars)
 
     if config.base.frequency == "1d":
         logger.DATETIME_FORMAT = "%Y-%m-%d"
@@ -287,16 +276,3 @@ def parse_persist_mode(persist_mode):
         return mapping[persist_mode]
     except KeyError:
         raise RuntimeError(_(u"unknown persist mode: {}").format(persist_mode))
-
-
-def parse_market(market):
-    assert isinstance(market, six.string_types)
-    mapping = {
-        "cn": MARKET.CN,
-        "hk": MARKET.HK
-    }
-
-    try:
-        return mapping[market.lower()]
-    except KeyError:
-        raise RuntimeError(_(u"unknown market type: {}".format(market)))
